<?php
/**
 * Widget API: Advanced Menu Widget Class
 *
 * @package WordPress
 * @subpackage Widgets
 * @since 4.4.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * Register Advanced Menu Widget
 */
add_action( 'widgets_init', function(){
    register_widget( 'YSD_Advanced_Menu_Widget' );
});

/**
 * Core class used to implement the Advanced Menu Widget.
 *
 * @since 3.0.0
 *
 * @see WP_Widget
 */
class YSD_Advanced_Menu_Widget extends WP_Widget {

	/**
	 * Sets up a new Advanced Menu Widget instance.
	 *
	 * @since 3.0.0
	 */
	public function __construct() {
		$widget_ops = array(
			'description' => __( 'Add an Advanced Menu Widget to display your menu items nicely.', YSD_AW_TEXT_DOMAIN ),
			'customize_selective_refresh' => true,
		);
		parent::__construct( 'ysd_advanced_nav_menu', __('Navigation Menu (AW)', YSD_AW_TEXT_DOMAIN), $widget_ops );
	}

	/**
	 * Outputs the content for the current Navigation Menu widget instance.
	 *
	 * @since 3.0.0
	 *
	 * @param array $args     Display arguments including 'before_title', 'after_title',
	 *                        'before_widget', and 'after_widget'.
	 * @param array $instance Settings for the current Navigation Menu widget instance.
	 */
	public function widget( $args, $instance ) {

        // Include Class Walker Navigation Menu
        include YSD_AW_PLUGIN_PATH . '/includes/class-walker-nav-menu.php';
        // Enqueue Style
        wp_enqueue_style( 'ysd_aw_style');
        // Enqueue Script
        wp_enqueue_script( 'ysd_aw_script');
        
		// Get menu
		$nav_menu = ! empty( $instance['nav_menu'] ) ? wp_get_nav_menu_object( $instance['nav_menu'] ) : false;

		if ( ! $nav_menu ) {
			return;
		}

		$title = ! empty( $instance['title'] ) ? $instance['title'] : '';

		/* This filter is documented in wp-includes/widgets/class-wp-widget-pages.php */
		
		$title = apply_filters( 'widget_title', $title, $instance, $this->id_base );
	
		echo $args['before_widget'];

        echo $title ? $args['before_title'] . $title . $args['after_title'] : '';
        
		$nav_menu_args = array(
			'fallback_cb'   => '',
			'menu'          => $nav_menu,
            'container_id'  => 'aw-advanced-nav-menu',
            'menu_class'    => 'menu ysd-advanced-widgets ysd-advanced-nav-menu',
			'walker'        => (class_exists('YSD_AW_Walker_Nav_Menu')) ? 'YSD_AW_Walker_Nav_Menu' : '',
		);

		/**
		 * Filters the arguments for the Navigation Menu widget.
		 *
		 * @since 4.2.0
		 * @since 4.4.0 Added the `$instance` parameter.
		 *
		 * @param array    $nav_menu_args {
		 *     An array of arguments passed to wp_nav_menu() to retrieve a navigation menu.
		 *
		 *     @type callable|bool $fallback_cb Callback to fire if the menu doesn't exist. Default empty.
		 *     @type mixed         $menu        Menu ID, slug, or name.
		 * }
		 * @param WP_Term  $nav_menu      Nav menu object for the current menu.
		 * @param array    $args          Display arguments for the current widget.
		 * @param array    $instance      Array of settings for the current widget.
		 */
        wp_nav_menu( apply_filters( 'widget_nav_menu_args', $nav_menu_args, $nav_menu, $args, $instance ) );

		echo $args['after_widget'];
		
	}

	/**
	 * Handles updating settings for the current Navigation Menu widget instance.
	 *
	 * @since 3.0.0
	 *
	 * @param array $new_instance New settings for this instance as input by the user via
	 *                            WP_Widget::form().
	 * @param array $old_instance Old settings for this instance.
	 * @return array Updated settings to save.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		if ( ! empty( $new_instance['title'] ) ) {
			$instance['title'] = sanitize_text_field( $new_instance['title'] );
		}
		if ( ! empty( $new_instance['nav_menu'] ) ) {
			$instance['nav_menu'] = (int) $new_instance['nav_menu'];
		}
		return $instance;
	}

	/**
	 * Outputs the settings form for the Navigation Menu widget.
	 *
	 * @since 3.0.0
	 *
	 * @param array $instance Current settings.
	 * @global WP_Customize_Manager $wp_customize
	 */
	public function form( $instance ) {

        global $wp_customize;
        
		$title = isset( $instance['title'] ) ? $instance['title'] : '';
		$nav_menu = isset( $instance['nav_menu'] ) ? $instance['nav_menu'] : '';

		// Get menus
		$menus = wp_get_nav_menus();

		// If no menus exists, direct the user to go and create some.
        if ( ! empty( $menus ) ) : ?>

            <div class="nav-menu-widget-form-controls">
                <p>
                    <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', YSD_AW_TEXT_DOMAIN ); ?></label>
                    <input type="text" class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo esc_attr( $title ); ?>"/>
                </p>
                <p>
                    <label for="<?php echo $this->get_field_id( 'nav_menu' ); ?>"><?php _e( 'Select Menu:', YSD_AW_TEXT_DOMAIN ); ?></label>
                    <select id="<?php echo $this->get_field_id( 'nav_menu' ); ?>" name="<?php echo $this->get_field_name( 'nav_menu' ); ?>">
                        <option value="0"><?php _e( '&mdash; Select &mdash;', YSD_AW_TEXT_DOMAIN ); ?></option>
                        <?php foreach ( $menus as $menu ) : ?>
                            <option value="<?php echo esc_attr( $menu->term_id ); ?>" <?php selected( $nav_menu, $menu->term_id ); ?>>
                                <?php echo esc_html( $menu->name ); ?>
                            </option>
                        <?php endforeach; ?>
                    </select>
                </p>
                <?php if ( $wp_customize instanceof WP_Customize_Manager ) : ?>
                    <?php if ( $nav_menu ) : ?>
                        <p class="edit-selected-nav-menu">
                            <button type="button" class="button"><?php _e( 'Edit Menu', YSD_AW_TEXT_DOMAIN ); ?></button>
                        </p>
                    <?php endif; ?>
                <?php endif; ?>
            </div>

        <?php else : ?>

            <p class="nav-menu-widget-no-menus-message">
                <?php
                if ( $wp_customize instanceof WP_Customize_Manager ) {
                    $url = 'javascript: wp.customize.panel( "nav_menus" ).focus();';
                } else {
                    $url = admin_url( 'nav-menus.php' );
                }
                ?>
                <?php echo sprintf( __( 'No menus have been created yet. <a href="%s">Create some</a>.', YSD_AW_TEXT_DOMAIN ), esc_attr( $url ) ); ?>
            </p>

        <?php
        endif;
	}
}
